-------------------------------------------
-- Author: Gregor "farhaven" Best"       --
-- Copyright 2009 Gregor Best            --
-------------------------------------------

require("obvious.basic_mpd")
require("obvious.battery")
require("obvious.clock")
require("obvious.cpu")
require("obvious.fs_usage")
require("obvious.io")
require("obvious.lib")
require("obvious.loadavg")
require("obvious.mem")
require("obvious.net")
require("obvious.popup_run_prompt")
require("obvious.umts")
require("obvious.volume_alsa")
require("obvious.volume_freebsd")
require("obvious.wlan")
require("obvious.temp_info")
require("obvious.keymap_switch")

module("obvious")

-- vim:ft=lua:ts=2:sw=2:sts=2:tw=80:et
